import logging
import numpy as np

from PySide6.QtCore import Qt, QAbstractTableModel, QSortFilterProxyModel

class PoseTableModel(QAbstractTableModel):
    column_header = ["frame", "track"]
    columns = ["frame", "track"]

    def __init__(self, dataframe, parent=None):
        super().__init__(parent)
        self._data = dataframe
        self._frames = self._data.frame.values
        self._tracks = self._data.track.values
        self._indices = self._data.index.values
        self._column_data = [self._frames, self._tracks]

    def columnCount(self, parent=None):
        return len(self.columns)

    def rowCount(self, parent=None):
        if self._data is not None:
            return len(self._data)
        else:
            return 0

    def data(self, index, role = ...):
        value = self._column_data[index.column()][index.row()]
        if role == Qt.ItemDataRole.DisplayRole:
            return str(value)
        elif role == Qt.ItemDataRole.UserRole:
            return value
        return None

    def headerData(self, section, orientation, role = ...):
        if role == Qt.ItemDataRole.DisplayRole:
            if orientation == Qt.Orientation.Horizontal:
                return self.column_header[section]
            else:
                return str(self._indices[section])
        else:
            return None

    def mapIdToRow(self, id):
        row = np.where(self._indices == id)[0]
        if len(row) == 0:
            return -1
        return row[0]


class FilterProxyModel(QSortFilterProxyModel):
    def __init__(self, parent=None):
        super().__init__(parent)
        self._range = None

    def setFilterRange(self, start, stop):
        logging.info("FilterProxyModel.setFilterRange set to range  %i , %i", start, stop)
        self._range = (start, stop)
        self.invalidateRowsFilter()

    def all(self):
        self._range = None

    def filterAcceptsRow(self, source_row, source_parent):
        if self._range is None:
            return True
        else:
            idx = self.sourceModel().index(source_row, 0, source_parent);
            val = self.sourceModel().data(idx, Qt.ItemDataRole.UserRole)
            print("filteracceptrows: ", val, self._range, val >= self._range[0] and val < self._range[1]  )
            return val >= self._range[0] and val < self._range[1]

    def filterAcceptsColumn(self, source_column, source_parent):
        return True
