from PySide6.QtWidgets import QHBoxLayout, QFrame, QLabel, QVBoxLayout
from PySide6.QtCore import Qt, Signal
from PySide6.QtGui import QFont, QImage, QPixmap


class TaskCard(QFrame):
    clicked = Signal()

    def __init__(self, name, icon, description, parent=None):
        super().__init__(parent)
        self.setMaximumSize(300, 200)
        self.setFrameShape(QFrame.Shape.Box)
        # self.setStyleSheet("border: 2px solid gray; border-radius: 15px; border-color: white;background-color: green;")
        font = QFont()
        font.setBold(True)
        font.setPointSize(18)

        name_label = QLabel(name)
        name_label.setFont(font)
        description_label = QLabel(description)
        description_label.setLineWidth(150)
        description_label.setWordWrap(True)
        icon_label = QLabel()
        icon_label.setFixedSize(150, 150)
        pixmap = QPixmap.fromImage(QImage(icon))
        scaled_pixmap = pixmap.scaled(icon_label.size(), Qt.AspectRatioMode.KeepAspectRatio, Qt.TransformationMode.SmoothTransformation)
        icon_label.setPixmap(scaled_pixmap)

        hbox = QHBoxLayout()
        hbox.addWidget(icon_label)
        hbox.addWidget(description_label)
        self.mousePressEvent = self._on_click

        lyt = QVBoxLayout()
        lyt.addWidget(name_label)
        lyt.addLayout(hbox)
        self.setLayout(lyt)

    def _on_click(self, event):
        self.clicked.emit()
